module Corraios

  module Containers

    class Base

      def initialize(attributes)
        attributes.each do |attr, value|
          send "#{attr}=", value.to_f
        end
      end

      def is_a?(type)
        self.class.name.split('::').last.downcase.to_sym == type
      end

      def to_package
        raise NotImplementedError
      end

      def to_query_params
        attributes_names.map do |attr|
          "#{self.class.query_attribute_name(attr)}=#{send(attr).round(2).to_s.gsub('.', ',')}"
        end.join('&') + "&nCdFormato=#{format}"
      end

      def format
        self.class.instance_variable_get('@format')
      end

      def attributes_names
        self.class.instance_variable_get('@attributes_restrictions').keys
      end

      def valid?
        attributes_names.all? do |attr|
          self.class.valid_field?(attr, send(attr))
        end
      end

      def assert_minimum_measures!
        attributes_names.each do |attr|
          send("#{attr}=", self.class.floor_for(attr, send(attr)))
        end
      end

      class << self

        def add_attribute name, restrictions, query_attribute_name = nil, round_value = true
          attributes_restrictions[name] = restrictions
          query_attribute_names[name] = query_attribute_name
          attr_accessor name
        end

        def set_format(format)
          @format = format
        end

        def valid_field?(attribute, value)
          attributes_restrictions[attribute] && attributes_restrictions[attribute].include?(value)
        end

        def attributes_restrictions
          @attributes_restrictions ||= {}
        end

        def query_attribute_names
          @query_attribute_names ||= {}
        end

        def valid_fields?(attributes)
          attributes.keys.size == attributes_restrictions.keys.size &&
            attributes.all? { |attribute, value| valid_field?(attribute, value) }
        end

        def floor_for(attribute, value)
          minimum_value = minimum_value_for(attribute)

          minimum_value < value ? value : minimum_value
        end

        def minimum_value_for(attribute)
          attributes_restrictions[attribute].first
        end

        def query_attribute_name(attribute_name)
          query_attribute_names[attribute_name]
        end

      end
    end
  end

end