module Corraios

  module Containers

    class Package < Base
      set_format 1
      add_attribute :weight, 0.0..30, 'nVlPeso'
      add_attribute :width, 11..105, 'nVlLargura'
      add_attribute :length, 16..105, 'nVlComprimento'
      add_attribute :height, 2..105, 'nVlAltura'

      def can_merge?(other)
        new_volume = self.volume + other.volume
        new_sides_size = (new_volume ** (1.0/3))

        attributes = { weight: self.weight + other.weight }
        [:width, :length, :height].each do |attr|
          attributes[attr] = self.class.floor_for attr, new_sides_size
        end

        self.class.valid_fields? attributes
      end

      def merge!(other)
        self.weight += other.weight
        self.width = self.length = self.height = ((self.volume + other.volume) ** (1.0/3))
        assert_minimum_measures!
      end

      def volume
        width * length * height
      end

      def to_package
        self
      end

      class << self

        def valid_fields?(attributes)
          super(attributes) &&
            (attributes[:width] + attributes[:height] + attributes[:length]) <= 200.0
        end
      end
    end
  end
end