module Corraios

  class Delivery
    attr_reader :name, :code
    attr_reader :value, :deadline, :error_message

    attr_reader :amount_of_posts

    def initialize(correios_response, service_name, service_code)
      @name = service_name
      @code = service_code
      @amount_of_posts = 1
      parse_response(correios_response)
    end

    def error?
      !!@error_message
    end

    def success?
      !error?
    end

    def merge!(other)
      raise CorraiosError unless self == other
      if success? && other.success?
        merge_when_success!(other)
      else
        merge_when_error!(other)
      end
    end

    def ==(other)
      @code == other.code
    end

    private

    def merge_when_success!(other)
      @value += other.value
      @deadline = [@deadline, other.deadline].max
    end

    def merge_when_error!(other)
      @value = 0.0
      @deadline = 0
      @error_message ||= other.error_message
    end

    def parse_response(response)
      service = validate_response(response)
      value = to_float(service['Valor'])
      if value > 0.0
        @value = value
        @deadline = service['PrazoEntrega'].to_i
      else
        @error_message = service['MsgErro'].strip
      end
    end

    def to_float(string)
      string.to_s.gsub(',', '.').to_f
    end

    def validate_response(response)
      if response['Servicos'] && response['Servicos']['cServico']
        response['Servicos']['cServico']
      else
        raise InvalidResponse
      end
    end
  end
end