module Corraios

  class Packer

    attr_reader :containers

    def initialize()
      @containers = []
    end

    def add_item(attributes)
      container = build_container(attributes)
      add_to_containers container
    end

    def total_weight
      @containers.inject(0) { |sum, item| sum + item.weight }.round(3)
    end

    def amount_of_posts
      @containers.size
    end

    private
    def build_container(attributes)
      container = infers_container(attributes)
      raise UnknownMeasures unless container
      container.new(attributes)
    end

    def add_to_containers(container)
      if @containers.empty?
        @containers << container
      else
        if container.is_a? :package
          convert_containers_into_package unless using_package?
          add_package(container)
        else
          add_envelope_or_roll container
        end
      end
    end

    def first_empty
      @containers.last
    end

    def add_envelope_or_roll(envelope)
      unless using_envelope? && try_merge(first_empty, envelope)
        convert_containers_into_package
        add_package(envelope.to_package)
      end
    end

    def convert_containers_into_package
      @containers = @containers.map &:to_package
    end

    def add_package(package)
      unless try_merge(first_empty, package)
        @containers << package
      end
    end

    def try_merge(container, new_container)
      if container.can_merge? new_container
        container.merge! new_container
        true
      end
    end

    def using_package?
      @containers.all? { |container| container.is_a? :package }
    end

    def using_envelope?
      @containers.size == 1 && @containers[0].is_a?(:envelope)
    end

    def pack_all_into_one_package(container)
      weight_sum = total_weight + container.weight
      measures = { height: 2, length: container.length, width: container.width }
      @containers = [ Containers::Package.new(measures.merge(weight: weight_sum)) ]
    end

    def infers_container(attributes)
      [Containers::Package, Containers::Envelope, Containers::Roll].find do |clazz|
        clazz.valid_fields?(attributes)
      end
    end
  end
end